<#
Company:        Dialog Verwaltungs-Data AG
Product:        Dialog G6
Purpose:        Create in Azure an App registration "DialogLohn Client"
Developer:      alo;ala
Create:         2025-03-20
Version:        2.1
Description:    
	Diese PowerShel Skript erstellt in Azure eine App registration "DialogLohn Client" mit alle benötigte Rechte gem.:
	http://confluence.dialog.ch:8090/display/GA/Azure+G6+Client+Aplikation+for+IMAP

	Voraussetzungen:
	
	1. Azure CLI muss installiert werden
	https://learn.microsoft.com/en-us/cli/azure/install-azure-cli
	
	2. Skript soll "Als Administrator" ausgeführt werden

	3. Damit auch Grant admin-consent in Azure funktioniert (der letzte Befehl), sollte die Person, die Skript ausführt ein Administrator auf dem Azure sein
#>

# Set-ExecutionPolicy -Scope Process -ExecutionPolicy Bypass

$appName = "DialogLohn Client"
$tryCount = 12;
$waitMilliseconds = 15000

function Test-LastExitCode {
	if ($LastExitCode -ne 0) {
		Write-Host "--> Error ($LastExitCode) --> exit" -ForegroundColor Red
	} else {
		Write-Host "--> Ok" -ForegroundColor Green
	}
	return $LastExitCode
}

function Waiting {
	Write-Output "- Waiting $waitMilliseconds Milliseconds..."
	Start-Sleep -Milliseconds $waitMilliseconds
}

function Write-Step {
	param(
		[string]$title,
		[string]$command
	)
	Write-Output ""
	Write-Host $title -ForegroundColor DarkCyan
	if ($command -ne "") {
		Write-Host $command -ForegroundColor Cyan
	}
}

function Test-Grant {
	param(
		[string[]]$grantsArray,
		[string]$name
	)
	$isExists = $grantsArray.Contains($name);
	if ($isExists) {
		Write-Host "- $name Ok" -ForegroundColor Green
	}
	else {
		Write-Host "- $name Missing" -ForegroundColor Red
	}
	return $isExists
}

Write-Output "**************************************** -= START =- ****************************************"

Write-Step "1. Login" "az login --allow-no-subscriptions"
az login --allow-no-subscriptions
if (Test-LastExitCode -ne 0) { return 1 }

Write-Step "2. Create App Registration '$appName'" "az ad app create --display-name '$appName' ..."
$createAppJsonOutput = az ad app create --display-name $appName --required-resource-accesses manifest.json --sign-in-audience AzureADMyOrg --is-fallback-public-client true --web-redirect-uris https://login.microsoftonline.com/common/oauth2/nativeclient
if (Test-LastExitCode -ne 0) { return 2 }

Write-Step "3. Read appId" ""
$createAppJson = $createAppJsonOutput | ConvertFrom-Json
$appId = $createAppJson.appId
$objId = $createAppJson.id
Write-Host "[$appName] has appId [$appId]" -ForegroundColor Green
if (Test-LastExitCode -ne 0) { return 3 }

Write-Step "4. Grand admin consent" "az ad app permission admin-consent --id $appId"
for ($count = 1; $count -le $tryCount; $count++) {
	Write-Output "- Try $count"	
	Waiting
	$result = az ad app permission admin-consent --id $appId 2>&1
	Write-Output $result
	
	if (Test-LastExitCode -ne 0) {
		# Write-Host "--> Error $LastExitCode" -ForegroundColor Red
		# 'Forbidden({"ClassName":"Microsoft.Portal.Framework.Exceptions.ClientException",'
		# 'Bad Request({"ClassName":"Microsoft.Portal.Framework.Exceptions.ClientException",'
		if ($result.ToString().Contains('Bad Request({"ClassName":"Microsoft.Portal.Framework.Exceptions.ClientException",')) { 
			Continue
		}
		else {
			Break;
		}		
	} else {
		Break
	}
}
if ($LastExitCode -ne 0) { return 4 }

Write-Step "5. Check grants admin consent" "az ad app permission list-grants --id $appId"
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
for ($count = 1; $count -le $tryCount; $count++) {
	Write-Output "- Try $count"	
	Waiting
	$grantsListOutput = az ad app permission list-grants --id $appId # b8009a96-9d2f-4f3b-9873-71b55427fff0
	
	if (Test-LastExitCode -ne 0) {
		Continue	
	} else {
		Break
	}
}
if ($LastExitCode -ne 0) { return 5 }
Waiting
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

Write-Output "$grantsListOutput"
$grantsListNamesString = $grantsListOutput | ConvertFrom-Json
# expected is: "Mail.Send (Delegated) Mail.Send (App)"
$scope = $grantsListNamesString[0].scope;
Write-Output "- grants admin consent scope: $scope"
#Mail.Send (Delegated) Mail.Send (App)
$grantsArray = $scope.Split(" ");

$SendMail_Grant = Test-Grant $grantsArray "Mail.Send"

if ($offline_access_Grant -and $email_Grant -and $SendMail_Grant) {
	Write-Host "--> Ok" -ForegroundColor Green
}
else {
	Write-Host "--> Error: Admin consent grants have a problem" -ForegroundColor Red
}

Write-Step "6. Get Results for DialogLohn" "------------------------------------------------------------------"
$tenantId = az account list --query [0].tenantId --output tsv
# $aadAppsecret01 = New-AzureADApplicationPasswordCredential -ObjectId $objectId -CustomKeyIdentifier "secret01"
# $aadAppsecret01 = az ad app credential reset --id $objectId --append --display-name 'secret01' --end-date '2030-12-31'

Write-Host "Application (client) Name   [$appName]" -ForegroundColor Green
# Write-Host "Application (client) ID:    [$appId]" -ForegroundColor Green
Write-Host "Directory (tenant) ID:      [$tenantId]" -ForegroundColor Green
# Write-Host "Client Secret:              [$aadAppsecret01]" -ForegroundColor Green


Write-Output "**************************************** -= FINISH =- ****************************************"
Return 0
